@@ -13,22 +13,23 @@ from __future__ import division
 # compatibility functions and utilities
 from ._utils import _supports_unicode, _environ_cols_wrapper, _range, _unich, \
     _term_move_up, _unicode, WeakSet, _basestring, _OrderedDict
+from ._monitor import TMonitor
 # native libraries
 import sys
 from numbers import Number
-from threading import Thread
 from time import time
-from time import sleep
 from contextlib import contextmanager
 # For parallelism safety
 import multiprocessing as mp
 import threading as th
-
+from warnings import warn
 
 __author__ = {"github.com/": ["noamraph", "obiwanus", "kmike", "hadim",
                               "casperdcl", "lrq3000"]}
 __all__ = ['tqdm', 'trange',
-           'TqdmTypeError', 'TqdmKeyError', 'TqdmDeprecationWarning']
+           'TqdmTypeError', 'TqdmKeyError', 'TqdmWarning',
+           'TqdmExperimentalWarning', 'TqdmDeprecationWarning',
+           'TqdmMonitorWarning']
 
 
 class TqdmTypeError(TypeError):
@@ -39,22 +40,42 @@ class TqdmKeyError(KeyError):
     pass
 
 
-class TqdmDeprecationWarning(Exception):
-    # not suppressed if raised
+class TqdmWarning(Warning):
+    """base class for all tqdm warnings.
+
+    Used for non-external-code-breaking errors, such as garbled printing.
+    """
     def __init__(self, msg, fp_write=None, *a, **k):
         if fp_write is not None:
-            fp_write("\nTqdmDeprecationWarning: " + str(msg).rstrip() + '\n')
+            fp_write("\n" + self.__class__.__name__ + ": " + str(msg).rstrip() + '\n')
         else:
-            super(TqdmDeprecationWarning, self).__init__(msg, *a, **k)
+            super(TqdmWarning, self).__init__(msg, *a, **k)
+
+class TqdmExperimentalWarning(TqdmWarning, FutureWarning):
+    """beta feature, unstable API and behaviour"""
+    pass
+
+class TqdmDeprecationWarning(TqdmWarning, DeprecationWarning):
+    # not suppressed if raised
+    pass
+
+
+class TqdmMonitorWarning(TqdmWarning, RuntimeWarning):
+    """tqdm monitor errors which do not affect external functionality"""
+    pass
 
 
 # Create global parallelism locks to avoid racing issues with parallel bars
 # works only if fork available (Linux, MacOSX, but not on Windows)
 try:
     mp_lock = mp.RLock()  # multiprocessing lock
-    th_lock = th.RLock()  # thread lock
+except ImportError:  # pragma: no cover
+    mp_lock = None
 except OSError:  # pragma: no cover
     mp_lock = None
+try:
+    th_lock = th.RLock()  # thread lock
+except OSError:  # pragma: no cover
     th_lock = None
 
 
@@ -84,80 +105,6 @@ class TqdmDefaultWriteLock(object):
         self.release()
 
 
-class TMonitor(Thread):
-    """
-    Monitoring thread for tqdm bars.
-    Monitors if tqdm bars are taking too much time to display
-    and readjusts miniters automatically if necessary.
-
-    Parameters
-    ----------
-    tqdm_cls  : class
-        tqdm class to use (can be core tqdm or a submodule).
-    sleep_interval  : fload
-        Time to sleep between monitoring checks.
-    """
-
-    # internal vars for unit testing
-    _time = None
-    _sleep = None
-
-    def __init__(self, tqdm_cls, sleep_interval):
-        Thread.__init__(self)
-        self.daemon = True  # kill thread when main killed (KeyboardInterrupt)
-        self.was_killed = False
-        self.woken = 0  # last time woken up, to sync with monitor
-        self.tqdm_cls = tqdm_cls
-        self.sleep_interval = sleep_interval
-        if TMonitor._time is not None:
-            self._time = TMonitor._time
-        else:
-            self._time = time
-        if TMonitor._sleep is not None:
-            self._sleep = TMonitor._sleep
-        else:
-            self._sleep = sleep
-        self.start()
-
-    def exit(self):
-        self.was_killed = True
-        # self.join()  # DO NOT, blocking event, slows down tqdm at closing
-        return self.report()
-
-    def run(self):
-        cur_t = self._time()
-        while True:
-            # After processing and before sleeping, notify that we woke
-            # Need to be done just before sleeping
-            self.woken = cur_t
-            # Sleep some time...
-            self._sleep(self.sleep_interval)
-            # Quit if killed
-            # if self.exit_event.is_set():  # TODO: should work but does not...
-            if self.was_killed:
-                return
-            # Then monitor!
-            # Acquire lock (to access _instances)
-            with self.tqdm_cls.get_lock():
-                cur_t = self._time()
-                # Check tqdm instances are waiting too long to print
-                for instance in self.tqdm_cls._instances:
-                    # Only if mininterval > 1 (else iterations are just slow)
-                    # and last refresh exceeded maxinterval
-                    if instance.miniters > 1 and \
-                            (cur_t - instance.last_print_t) >= \
-                            instance.maxinterval:
-                        # force bypassing miniters on next iteration
-                        # (dynamic_miniters adjusts mininterval automatically)
-                        instance.miniters = 1
-                        # Refresh now! (works only for manual tqdm)
-                        instance.refresh(nolock=True)
-
-    def report(self):
-        # return self.is_alive()  # TODO: does not work...
-        return not self.was_killed
-
-
 class tqdm(object):
     """
     Decorate an iterable object, returning an iterator which acts exactly
@@ -455,20 +402,22 @@ class tqdm(object):
         # Create the monitoring thread
         if cls.monitor_interval and (cls.monitor is None or not
                                      cls.monitor.report()):
-            cls.monitor = TMonitor(cls, cls.monitor_interval)
+            try:
+                cls.monitor = TMonitor(cls, cls.monitor_interval)
+            except Exception as e:  # pragma: nocover
+                warn("tqdm:disabling monitor support"
+                     " (monitor_interval = 0) due to:\n" + str(e),
+                     TqdmMonitorWarning)
+                cls.monitor_interval = 0
         # Return the instance
         return instance
 
     @classmethod
     def _get_free_pos(cls, instance=None):
-        """ Skips specified instance """
-        try:
-            return max(inst.pos for inst in cls._instances
-                       if inst is not instance) + 1
-        except ValueError as e:
-            if "arg is an empty sequence" in str(e):
-                return 0
-            raise  # pragma: no cover
+        """Skips specified instance"""
+        positions = set(abs(inst.pos) for inst in cls._instances
+                        if inst is not instance)
+        return min(set(range(len(positions) + 1)).difference(positions))
 
     @classmethod
     def _decr_instances(cls, instance):
@@ -476,23 +425,27 @@ class tqdm(object):
         Remove from list and reposition other bars
         so that newer bars won't overlap previous bars
         """
-        try:  # in case instance was explicitly positioned, it won't be in set
-            with cls._lock:
+        with cls._lock:
+            try:
                 cls._instances.remove(instance)
+            except KeyError:
+                if not instance.gui:  # pragma: no cover
+                    raise
+            else:
                 for inst in cls._instances:
-                    if inst.pos > instance.pos:
+                    # negative `pos` means fixed
+                    if inst.pos > abs(instance.pos):
                         inst.pos -= 1
-            # Kill monitor if no instances are left
-            if not cls._instances and cls.monitor:
-                try:
-                    cls.monitor.exit()
-                    del cls.monitor
-                except AttributeError:  # pragma: nocover
-                    pass
-                else:
-                    cls.monitor = None
-        except KeyError:
-            pass
+                        # TODO: check this doesn't overwrite another fixed bar
+        # Kill monitor if no instances are left
+        if not cls._instances and cls.monitor:
+            try:
+                cls.monitor.exit()
+                del cls.monitor
+            except AttributeError:  # pragma: nocover
+                pass
+            else:
+                cls.monitor = None
 
     @classmethod
     def write(cls, s, file=None, end="\n", nolock=False):
@@ -534,7 +487,6 @@ class tqdm(object):
                 inst.refresh(nolock=True)
         if not nolock:
             cls._lock.release()
-        # TODO: make list of all instances incl. absolutely positioned ones?
 
     @classmethod
     def set_lock(cls, lock):
@@ -597,21 +549,21 @@ class tqdm(object):
                     Data (may be grouped).
                 func  : function
                     To be applied on the (grouped) data.
-                *args, *kwargs  : optional
+                **kwargs  : optional
                     Transmitted to `df.apply()`.
                 """
+
                 # Precompute total iterations
                 total = getattr(df, 'ngroups', None)
                 if total is None:  # not grouped
-                    if isinstance(df, Series):
+                    if df_function == 'applymap':
+                        total = df.size
+                    elif isinstance(df, Series):
                         total = len(df)
-                    else:
-                        if kwargs.get('axis') == 1:
-                            total = len(df)
-                        else:
-                            total = df.size // len(df)
-                else:
-                    total += 1  # pandas calls update once too many
+                    else:  # DataFrame or Panel
+                        axis = kwargs.get('axis', 0)
+                        # when axis=0, total is shape[axis1]
+                        total = df.size // df.shape[axis]
 
                 # Init bar
                 if deprecated_t[0] is not None:
@@ -620,14 +572,27 @@ class tqdm(object):
                 else:
                     t = tclass(*targs, total=total, **tkwargs)
 
+                if len(args) > 0:
+                    # *args intentionally not supported (see #244, #299)
+                    TqdmDeprecationWarning(
+                        "Except func, normal arguments are intentionally" +
+                        " not supported by" +
+                        " `(DataFrame|Series|GroupBy).progress_apply`." +
+                        " Use keyword arguments instead.",
+                        fp_write=getattr(t.fp, 'write', sys.stderr.write))
+
                 # Define bar updating wrapper
                 def wrapper(*args, **kwargs):
-                    t.update()
+                    # update tbar correctly
+                    # it seems `pandas apply` calls `func` twice
+                    # on the first column/row to decide whether it can
+                    # take a fast or slow code path; so stop when t.total==t.n
+                    t.update(n=1 if t.total and t.n < t.total else 0)
                     return func(*args, **kwargs)
 
-                # Apply the provided function (in *args and **kwargs)
+                # Apply the provided function (in **kwargs)
                 # on the df using our wrapper (which provides bar updating)
-                result = getattr(df, df_function)(wrapper, *args, **kwargs)
+                result = getattr(df, df_function)(wrapper, **kwargs)
 
                 # Close bar and return pandas calculation result
                 t.close()
@@ -857,19 +822,18 @@ class tqdm(object):
         # not overwrite the outer progress bar
         if position is None:
             self.pos = self._get_free_pos(self)
-        else:
-            self.pos = position
-            self._instances.remove(self)
+        else:  # mark fixed positions as negative
+            self.pos = -position
 
         if not gui:
             # Initialize the screen printer
             self.sp = self.status_printer(self.fp)
             with self._lock:
                 if self.pos:
-                    self.moveto(self.pos)
+                    self.moveto(abs(self.pos))
                 self.sp(self.__repr__(elapsed=0))
                 if self.pos:
-                    self.moveto(-self.pos)
+                    self.moveto(-abs(self.pos))
 
         # Init the time counter
         self.last_print_t = self._time()
@@ -902,13 +866,13 @@ class tqdm(object):
             self.bar_format, self.postfix, self.unit_divisor)
 
     def __lt__(self, other):
-        return self.pos < other.pos
+        return abs(self.pos) < abs(other.pos)
 
     def __le__(self, other):
         return (self < other) or (self == other)
 
     def __eq__(self, other):
-        return self.pos == other.pos
+        return abs(self.pos) == abs(other.pos)
 
     def __ne__(self, other):
         return not (self == other)
@@ -974,11 +938,11 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
                         self.n = n
                         with self._lock:
                             if self.pos:
-                                self.moveto(self.pos)
+                                self.moveto(abs(self.pos))
                             # Print bar update
                             sp(self.__repr__())
                             if self.pos:
-                                self.moveto(-self.pos)
+                                self.moveto(-abs(self.pos))
 
                         # If no `miniters` was specified, adjust automatically
                         # to the max iteration rate seen so far between 2 prints
@@ -1063,13 +1027,13 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
                 with self._lock:
                     if self.pos:
-                        self.moveto(self.pos)
+                        self.moveto(abs(self.pos))
 
                     # Print bar update
                     self.sp(self.__repr__())
 
                     if self.pos:
-                        self.moveto(-self.pos)
+                        self.moveto(-abs(self.pos))
 
                 # If no `miniters` was specified, adjust automatically to the
                 # maximum iteration rate seen so far between two prints.
@@ -1108,7 +1072,7 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
         self.disable = True
 
         # decrement instance pos and remove from internal set
-        pos = self.pos
+        pos = abs(self.pos)
         self._decr_instances(self)
 
         # GUI mode
@@ -1228,10 +1192,10 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
         if not nolock:
             self._lock.acquire()
-        self.moveto(self.pos)
+        self.moveto(abs(self.pos))
         self.sp('')
         self.fp.write('\r')  # place cursor back at the beginning of line
-        self.moveto(-self.pos)
+        self.moveto(-abs(self.pos))
         if not nolock:
             self._lock.release()
 
@@ -1244,9 +1208,9 @@ Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`
 
         if not nolock:
             self._lock.acquire()
-        self.moveto(self.pos)
+        self.moveto(abs(self.pos))
         self.sp(self.__repr__())
-        self.moveto(-self.pos)
+        self.moveto(-abs(self.pos))
         if not nolock:
             self._lock.release()
 
